@@ -136,7 +136,7 @@ std::string as_std_string(PyObject* str) {
 
   // conver to bytes if its unicode
   PyObjectPtr pStr;
-  if (PyUnicode_Check(str)) {
+  if (PyUnicode_Check(str) || isPyArrayScalar(str)) {
     str = PyUnicode_AsBytes(str);
     pStr.assign(str);
   }
@@ -192,6 +192,75 @@ bool has_null_bytes(PyObject* str) {
   }
 }
 
+// helpers to narrow python array type to something convertable from R,
+// guaranteed to return NPY_BOOL, NPY_LONG, NPY_DOUBLE, or NPY_CDOUBLE
+// (throws an exception if it's unable to return one of these types)
+int narrow_array_typenum(int typenum) {
+
+  switch(typenum) {
+  // logical
+  case NPY_BOOL:
+    typenum = NPY_BOOL;
+    break;
+    // integer
+  case NPY_BYTE:
+  case NPY_UBYTE:
+  case NPY_SHORT:
+  case NPY_USHORT:
+  case NPY_INT:
+    typenum = NPY_LONG;
+    break;
+    // double
+  case NPY_UINT:
+  case NPY_ULONG:
+  case NPY_ULONGLONG:
+  case NPY_LONG:
+  case NPY_LONGLONG:
+  case NPY_HALF:
+  case NPY_FLOAT:
+  case NPY_DOUBLE:
+    typenum = NPY_DOUBLE;
+    break;
+
+    // complex
+  case NPY_CFLOAT:
+  case NPY_CDOUBLE:
+    typenum = NPY_CDOUBLE;
+    break;
+
+
+    // string/object (leave these alone)
+  case NPY_STRING:
+  case NPY_UNICODE:
+  case NPY_OBJECT:
+    break;
+
+    // unsupported
+  default:
+    stop("Conversion from numpy array type %d is not supported", typenum);
+    break;
+  }
+
+  return typenum;
+}
+
+int narrow_array_typenum(PyArrayObject* array) {
+  return narrow_array_typenum(PyArray_TYPE(array));
+}
+
+int narrow_array_typenum(PyArray_Descr* descr) {
+  return narrow_array_typenum(descr->type_num);
+}
+
+bool is_numpy_str(PyObject* x) {
+  if (!isPyArrayScalar(x))
+    return false; // ndarray or other, not string
+
+  PyArray_DescrPtr descrPtr(PyArray_DescrFromScalar(x));
+  int typenum = narrow_array_typenum(descrPtr);
+  return (typenum == NPY_STRING || typenum == NPY_UNICODE);
+}
+
 bool is_python_str(PyObject* x) {
 
   if (PyUnicode_Check(x))
@@ -203,6 +272,9 @@ bool is_python_str(PyObject* x) {
   else if (!is_python3() && PyString_Check(x) && !has_null_bytes(x))
     return true;
 
+  else if (is_numpy_str(x))
+    return true;
+
   else
     return false;
 }
@@ -512,67 +584,6 @@ CharacterVector py_tuple_to_character(PyObject* tuple) {
   return vec;
 }
 
-// helpers to narrow python array type to something convertable from R,
-// guaranteed to return NPY_BOOL, NPY_LONG, NPY_DOUBLE, or NPY_CDOUBLE
-// (throws an exception if it's unable to return one of these types)
-int narrow_array_typenum(int typenum) {
-
-  switch(typenum) {
-  // logical
-  case NPY_BOOL:
-    typenum = NPY_BOOL;
-    break;
-    // integer
-  case NPY_BYTE:
-  case NPY_UBYTE:
-  case NPY_SHORT:
-  case NPY_USHORT:
-  case NPY_INT:
-    typenum = NPY_LONG;
-    break;
-    // double
-  case NPY_UINT:
-  case NPY_ULONG:
-  case NPY_ULONGLONG:
-  case NPY_LONG:
-  case NPY_LONGLONG:
-  case NPY_HALF:
-  case NPY_FLOAT:
-  case NPY_DOUBLE:
-    typenum = NPY_DOUBLE;
-    break;
-
-    // complex
-  case NPY_CFLOAT:
-  case NPY_CDOUBLE:
-    typenum = NPY_CDOUBLE;
-    break;
-
-
-    // string/object (leave these alone)
-  case NPY_STRING:
-  case NPY_UNICODE:
-  case NPY_OBJECT:
-    break;
-
-    // unsupported
-  default:
-    stop("Conversion from numpy array type %d is not supported", typenum);
-    break;
-  }
-
-  return typenum;
-}
-
-int narrow_array_typenum(PyArrayObject* array) {
-  return narrow_array_typenum(PyArray_TYPE(array));
-}
-
-int narrow_array_typenum(PyArray_Descr* descr) {
-  return narrow_array_typenum(descr->type_num);
-}
-
-
 void set_string_element(SEXP rArray, int i, PyObject* pyStr) {
   std::string str = as_std_string(pyStr);
   cetype_t ce = PyUnicode_Check(pyStr) ? CE_UTF8 : CE_NATIVE;
@@ -984,6 +995,21 @@ PyObject* r_to_py(RObject x, bool convert) {
            "converted");
     }
 
+    int flags = NPY_ARRAY_FARRAY_RO;
+
+    // because R logical vectors are just ints under the
+    // hood, we need to explicitly construct a boolean
+    // vector for our Python array. note that the created
+    // array will own the data so we do not free it after
+    if (typenum == NPY_BOOL) {
+      R_xlen_t n = XLENGTH(sexp);
+      bool* converted = (bool*) PyArray_malloc(n * sizeof(bool));
+      for (R_xlen_t i = 0; i < n; i++)
+        converted[i] = LOGICAL(sexp)[i];
+      data = converted;
+      flags |= NPY_ARRAY_OWNDATA;
+    }
+
     // create the matrix
     PyObject* array = PyArray_New(&PyArray_Type,
                                    nd,
@@ -992,7 +1018,7 @@ PyObject* r_to_py(RObject x, bool convert) {
                                    NULL,
                                    data,
                                    0,
-                                   NPY_ARRAY_FARRAY_RO,
+                                   flags,
                                    NULL);
 
     // check for error
